@@ -1,4 +1,4 @@
-use crate::{common::*, FromCv, TryFromCv};
+use crate::{common::*, FromCv, TryFromCv, TryIntoCv};
 use nalgebra::{self as na, geometry as geo};
 use opencv::{calib3d, core as core_cv, prelude::*};
 
@@ -15,14 +15,22 @@ impl TryFromCv<OpenCvPose<&core_cv::Point3d>> for geo::Isometry3<f64> {
     type Error = Error;
 
     fn try_from_cv(pose: OpenCvPose<&core_cv::Point3d>) -> Result<Self> {
-        let OpenCvPose { rvec, tvec } = pose;
+        (&pose).try_into_cv()
+    }
+}
+
+impl TryFromCv<&OpenCvPose<&core_cv::Point3d>> for geo::Isometry3<f64> {
+    type Error = Error;
+
+    fn try_from_cv(pose: &OpenCvPose<&core_cv::Point3d>) -> Result<Self> {
+        let OpenCvPose { rvec, tvec } = *pose;
         let rotation = {
             let rvec_mat = {
                 let core_cv::Point3_ { x, y, z, .. } = *rvec;
                 core_cv::Mat::from_slice(&[x, y, z])?
             };
             let mut rotation_mat = core_cv::Mat::zeros(3, 3, core_cv::CV_64FC1)?.to_mat()?;
-            calib3d::rodrigues(&rvec_mat, &mut rotation_mat, &mut core_cv::no_array()?)?;
+            calib3d::rodrigues(&rvec_mat, &mut rotation_mat, &mut core_cv::Mat::default())?;
             let rotation_matrix: na::Matrix3<f64> = TryFromCv::try_from_cv(rotation_mat)?;
             geo::UnitQuaternion::from_matrix(&rotation_matrix)
         };
@@ -58,10 +66,18 @@ impl TryFromCv<OpenCvPose<&core_cv::Mat>> for geo::Isometry3<f64> {
     type Error = Error;
 
     fn try_from_cv(from: OpenCvPose<&core_cv::Mat>) -> Result<Self> {
+        (&from).try_into_cv()
+    }
+}
+
+impl TryFromCv<&OpenCvPose<&core_cv::Mat>> for geo::Isometry3<f64> {
+    type Error = Error;
+
+    fn try_from_cv(from: &OpenCvPose<&core_cv::Mat>) -> Result<Self> {
         let OpenCvPose {
             rvec: rvec_mat,
             tvec: tvec_mat,
-        } = from;
+        } = *from;
         let rvec = core_cv::Point3d::try_from_cv(rvec_mat)?;
         let tvec = core_cv::Point3d::try_from_cv(tvec_mat)?;
         let isometry = TryFromCv::try_from_cv(OpenCvPose { rvec, tvec })?;
@@ -104,7 +120,7 @@ where
             let rotation_mat =
                 core_cv::Mat::try_from_cv(rotation.to_rotation_matrix().into_inner())?;
             let mut rvec_mat = core_cv::Mat::zeros(3, 1, core_cv::CV_64FC1)?.to_mat()?;
-            calib3d::rodrigues(&rotation_mat, &mut rvec_mat, &mut core_cv::no_array()?)?;
+            calib3d::rodrigues(&rotation_mat, &mut rvec_mat, &mut core_cv::Mat::default())?;
             let rvec = core_cv::Point3_::new(
                 *rvec_mat.at_2d::<T>(0, 0)?,
                 *rvec_mat.at_2d::<T>(1, 0)?,
@@ -143,7 +159,7 @@ impl TryFromCv<&geo::Isometry3<f64>> for OpenCvPose<core_cv::Mat> {
             let rotation_mat: Mat =
                 TryFromCv::try_from_cv(rotation.to_rotation_matrix().into_inner())?;
             let mut rvec_mat = core_cv::Mat::zeros(3, 1, core_cv::CV_64FC1)?.to_mat()?;
-            calib3d::rodrigues(&rotation_mat, &mut rvec_mat, &mut core_cv::no_array()?)?;
+            calib3d::rodrigues(&rotation_mat, &mut rvec_mat, &mut core_cv::Mat::default())?;
             rvec_mat
         };
         let tvec = core_cv::Mat::from_slice(&[translation.x, translation.y, translation.z])?;
@@ -173,7 +189,7 @@ impl TryFromCv<&geo::Isometry3<f32>> for OpenCvPose<core_cv::Mat> {
         let rvec = {
             let rotation_mat = Mat::try_from_cv(rotation.to_rotation_matrix().into_inner())?;
             let mut rvec_mat = Mat::zeros(3, 1, core_cv::CV_32FC1)?.to_mat()?;
-            calib3d::rodrigues(&rotation_mat, &mut rvec_mat, &mut core_cv::no_array()?)?;
+            calib3d::rodrigues(&rotation_mat, &mut rvec_mat, &mut core_cv::Mat::default())?;
             rvec_mat
         };
         let tvec = Mat::from_slice(&[translation.x, translation.y, translation.z])?;
@@ -469,7 +485,7 @@ mod tests {
         let output = core_cv::Mat::try_from_cv(input)?;
         let output_shape = output.size()?;
         ensure!(
-            output.channels()? == 1
+            output.channels() == 1
                 && nrows == output_shape.height as usize
                 && ncols == output_shape.width as usize,
             "the shape does not match"
